@@ -210,12 +210,17 @@ class KnapsackDataset(object):
         # storage used are cheaper than I thought.
         # Can probably use those.
         filename = Path(full_src_file_path.name)
-        if isinstance(src, B2Store):
+        if isinstance(src, B2Store) and isinstance(dst, FSStore):
             data = b""
-            src.download_file(
-                b2_file_path=full_src_file_path,
-                local_bucket_path=src.root / Path(bucket_url) / filename
-            )
+            # TODO: converting remote url to local only works here because B2Store is always used remote.
+            storage_url = dst.convert_remote_url_to_local(bucket_url)
+
+            local_file_path = dst.root / storage_url / filename
+            if not dst._file_exists(local_file_path):
+                src.download_file(
+                    b2_file_path=full_src_file_path,
+                    local_bucket_path=local_file_path
+                )
         elif isinstance(dst, B2Store):
             data = src.read_bytes_from_file(full_src_file_path)
             dst.upload_file(
@@ -387,6 +392,7 @@ class KnapsackDataset(object):
         response = self._close_op(
             op, transaction_id, new_repro_tag, blueprints, bytes_used=self.total_bytes_used,
         )
+        self.total_bytes_used = 0
         # if response is not None:
             # blueprints = Blueprints.from_json(response.json())
             # if blueprints is not None:
@@ -408,6 +414,7 @@ class KnapsackDataset(object):
         """
         self.repro_tag = self._run_op(Operation.STORE, self.kd_builder, dst_knapsack_name)
         self.kd_builder = None
+        return self.repro_tag
 
     async def _merge_request(
         self,
@@ -552,9 +559,7 @@ class KnapsackDataset(object):
         return self.get_total_num_data_points()
 
     def get_total_num_data_points(self) -> int:
-        blueprints = self._launch_asyncio(
-            self.current_local_knapsack.read_blueprints(self.org_name, self.name)
-        )
+        blueprints = self.current_local_knapsack.read_blueprints(self.org_name, self.name)
         blueprint = blueprints.get_blueprint(self.repro_tag)
         if blueprint is None:
             return 0
@@ -578,9 +583,7 @@ class KnapsackDataset(object):
         # TODO: optimize this function.
         if i >= len(self):
             i = i % len(self)
-        blueprints = self._launch_asyncio(
-            self.current_local_knapsack.read_blueprints(self.org_name, self.name)
-        )
+        blueprints = self.current_local_knapsack.read_blueprints(self.org_name, self.name)
         data_point_indices = blueprints.get_all_dp_indices_for_tag(self.repro_tag)
         data_point_idx = data_point_indices[i]
         return self.current_local_knapsack.get_data_by_idx(
